<?php

namespace Drupal\proc\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultAllowed;
use Drupal\Core\Access\AccessResultForbidden;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Session\AccountProxy;
use Drupal\proc\Entity\Proc;
use Drupal\proc\ProcInterface;
use Drupal\proc\Traits\ProcRecipientTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller.
 */
class ProcController extends ControllerBase {

  use ProcRecipientTrait;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * The current path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected CurrentPathStack $currentPath;

  /**
   * ProcController controller.
   *
   * @param \Drupal\Core\Session\AccountProxy $current_user
   *   The file repository.
   * @param \Drupal\Core\Path\CurrentPathStack $current_path
   *   The current path.
   */
  public function __construct(AccountProxy $current_user, CurrentPathStack $current_path) {
    $this->currentUser = $current_user;
    $this->currentPath = $current_path;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The Drupal service container.
   *
   * @return static
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('current_user'),
      $container->get('path.current')
    );
  }

  /**
   * Checks whether the user is a possible recipient for a PGP message.
   *
   * @param string $users
   *   User IDs as CSV.
   *
   * @return \Drupal\Core\Access\AccessResultForbidden|AccessResultAllowed
   *   Controller response.
   *
   * @SuppressWarnings(PHPMD.LongVariable)
   * @SuppressWarnings(PHPMD.StaticAccess)
   */
  public function checkRecipientsCsvController(string $users): AccessResultForbidden|AccessResultAllowed {
    $rejected_recipient_candidate = $this->checkRecipientsCsv($users);
    $auto_filter_recipients = $this->config('proc.settings')->get('proc-auto-filter-recipients') ?? 0;
    switch ($auto_filter_recipients) {
      case ProcInterface::STRATEGY_RESTRICTIVE:
        if (!is_int($rejected_recipient_candidate) && $rejected_recipient_candidate) {
          return AccessResult::allowed();
        }
        break;

      case ProcInterface::STRATEGY_PERMISSIVE:
        return AccessResult::allowed();
    }
    return AccessResult::forbidden(
      $this->t('User account %rejected_recipient_candidate does not exist or user is not a holder of PGP keys.', [
        '%rejected_recipient_candidate' => $rejected_recipient_candidate,
      ]));
  }

  /**
   * Checks whether the current user is listed as a recipient of
   * the proc entity.
   *
   * @SuppressWarnings(PHPMD.CyclomaticComplexity)
   * @SuppressWarnings(PHPMD.StaticAccess)
   */
  public function checkCurrentUserIsRecipient(): AccessResultForbidden|AccessResultAllowed {
    $current_user_id = $this->currentUser->id();
    $current_path = $this->currentPath->getPath();
    $proc_id = $this->extractProcIdFromPath($current_path);

    if (!$proc_id || !is_numeric($proc_id)) {
      return AccessResult::forbidden();
    }

    $proc = Proc::load($proc_id);
    if (!$proc instanceof Proc) {
      return AccessResult::forbidden();
    }

    // Check if the current user is the owner with permission or an admin.
    if ($this->isUserOwnerOrAdmin($proc, $current_user_id)) {
      return AccessResult::allowed();
    }

    // Check if the current user is a recipient.
    if ($this->isUserRecipient($proc, $current_user_id)) {
      return AccessResult::allowed();
    }

    return AccessResult::forbidden();
  }

  /**
   * Extracts the proc ID from given path.
   *
   * @param string $path
   *   The current path.
   *
   * @return int|null
   *   The proc ID or null if not found.
   */
  private function extractProcIdFromPath(string $path): ?int {
    $path = str_replace('/proc/', '', $path);
    $proc_id = strtok($path, '/');
    return is_numeric($proc_id) ? (int) $proc_id : null;
  }

  /**
   * Checks if the current user is owner of the proc or an admin.
   *
   * @param Proc $proc
   *   The proc entity.
   * @param int $current_user_id
   *   The current user ID.
   *
   * @return bool
   *   True if the user is the owner and has 'edit proc entity' permission or is
   *   an admin, false otherwise.
   */
  private function isUserOwnerOrAdmin(Proc $proc, int $current_user_id): bool {
    return ($proc->getOwnerId() == $current_user_id && $this->currentUser->hasPermission('edit proc entity')) ||
      $this->currentUser->hasPermission('administer proc entity');
  }

  /**
   * Checks if the current user is a recipient of the proc.
   *
   * @param Proc $proc
   *   The proc entity.
   * @param int $current_user_id
   *   The current user ID.
   *
   * @return bool
   *   True if the user is a recipient, false otherwise.
   */
  private function isUserRecipient(Proc $proc, int $current_user_id): bool {
    $recipients = $proc->get('field_recipients_set')->getValue();
    foreach ($recipients as $recipient) {
      if ($recipient['target_id'] == $current_user_id) {
        return true;
      }
    }
    return false;
  }

}
